package cn.iocoder.yudao.framework.common.thealgorithms.backtracking;

import java.util.ArrayList;
import java.util.List;

///**
// * This class generates all valid combinations of parentheses for a given number of pairs using backtracking.
// */
//public final class ParenthesesGenerator {
//    private ParenthesesGenerator() {
//    }
//
//    /**
//     * Generates all valid combinations of parentheses for a given number of pairs.
//     *
//     * @param n The number of pairs of parentheses.
//     * @return A list of strings representing valid combinations of parentheses.
//     * @throws IllegalArgumentException if n is less than 0.
//     */
//    public static List<String> generateParentheses(final int n) {
//        if (n < 0) {
//            throw new IllegalArgumentException("The number of pairs of parentheses cannot be nagative");
//        }
//        List<String> result = new ArrayList<>();
//        generateParenthesesHelper(result, "", 0, 0, n);
//        return result;
//    }
//
//    /**
//     * Helper function for generating all valid combinations of parentheses recursively.
//     *
//     * @param result  The list to store valid combinations.
//     * @param current The current combination being formed.
//     * @param open    The number of open parentheses.
//     * @param close   The number of closed parentheses.
//     * @param n       The total number of pairs of parentheses.
//     */
//    private static void generateParenthesesHelper(List<String> result, final String current, final int open, final int close, final int n) {
//        if (current.length() == n * 2) {
//            result.add(current);
//            return;
//        }
//        if (open < n) {
//            generateParenthesesHelper(result, current + "(", open + 1, close, n);
//        }
//        if (close < open) {
//            generateParenthesesHelper(result, current + ")", open, close + 1, n);
//        }
//    }
//}
//

/**
 * 这个类使用回溯法为给定数量的括号对生成所有有效的括号组合。
 */
public final class ParenthesesGenerator {
    private ParenthesesGenerator() {
    }

    /**
     * 为给定数量的括号对生成所有有效的括号组合。
     *
     * @param n 括号对的数量。
     * @return 表示有效括号组合的字符串列表。
     * @throws IllegalArgumentException 如果n小于0。
     */
    public static List<String> generateParentheses(final int n) {
        if (n < 0) {
            throw new IllegalArgumentException("括号对的数量不能为负数");
        }
        List<String> result = new ArrayList<>();
        generateParenthesesHelper(result, "", 0, 0, n);
        return result;
    }

    /**
     * 递归生成所有有效括号组合的辅助函数。
     *
     * @param result  用于存储有效组合的列表。
     * @param current 正在形成的当前组合。
     * @param open    开括号的数量。
     * @param close   闭括号的数量。
     * @param n       括号对的总数。
     */
    private static void generateParenthesesHelper(List<String> result, final String current, final int open, final int close, final int n) {
        if (current.length() == n * 2) {
            result.add(current);
            return;
        }
        if (open < n) {
            generateParenthesesHelper(result, current + "(", open + 1, close, n);
        }
        if (close < open) {
            generateParenthesesHelper(result, current + ")", open, close + 1, n);
        }
    }
}
