package cn.iocoder.yudao.framework.common.thealgorithms.backtracking;

import java.util.LinkedList;
import java.util.List;

///**
// * Finds all permutations of given array
// * @author Alan Piao (<a href="https://github.com/cpiao3">Git-Alan Piao</a>)
// */
//public final class Permutation {
//    private Permutation() {
//    }
//
//    /**
//     * Find all permutations of given array using backtracking
//     * @param arr the array.
//     * @param <T> the type of elements in the array.
//     * @return a list of all permutations.
//     */
//    public static <T> List<T[]> permutation(T[] arr) {
//        T[] array = arr.clone();
//        List<T[]> result = new LinkedList<>();
//        backtracking(array, 0, result);
//        return result;
//    }
//
//    /**
//     * Backtrack all possible orders of a given array
//     * @param arr the array.
//     * @param index the starting index.
//     * @param result the list contains all permutations.
//     * @param <T> the type of elements in the array.
//     */
//    private static <T> void backtracking(T[] arr, int index, List<T[]> result) {
//        if (index == arr.length) {
//            result.add(arr.clone());
//        }
//        for (int i = index; i < arr.length; i++) {
//            swap(index, i, arr);
//            backtracking(arr, index + 1, result);
//            swap(index, i, arr);
//        }
//    }
//
//    /**
//     * Swap two element for a given array
//     * @param a first index
//     * @param b second index
//     * @param arr the array.
//     * @param <T> the type of elements in the array.
//     */
//    private static <T> void swap(int a, int b, T[] arr) {
//        T temp = arr[a];
//        arr[a] = arr[b];
//        arr[b] = temp;
//    }
//}

/**
 * 查找给定数组的所有排列
 * @author Alan Piao (Git-Alan Piao)
 */
public final class Permutation {
    private Permutation() {
    }

    /**
     * 使用回溯法找到给定数组的所有排列
     * @param arr 数组。
     * @param <T> 数组中元素的类型。
     * @return 所有排列的列表。
     */
    public static <T> List<T[]> permutation(T[] arr) {
        T[] array = arr.clone();
        List<T[]> result = new LinkedList<>();
        backtracking(array, 0, result);
        return result;
    }

    /**
     * 回溯给定数组的所有可能顺序
     * @param arr 数组。
     * @param index 起始索引。
     * @param result 包含所有排列的列表。
     * @param <T> 数组中元素的类型。
     */
    private static <T> void backtracking(T[] arr, int index, List<T[]> result) {
        if (index == arr.length) {
            result.add(arr.clone());
        }
        for (int i = index; i < arr.length; i++) {
            swap(index, i, arr);
            backtracking(arr, index + 1, result);
            swap(index, i, arr);
        }
    }

    /**
     * 交换给定数组的两个元素
     * @param a 第一个索引
     * @param b 第二个索引
     * @param arr 数组。
     * @param <T> 数组中元素的类型。
     */
    private static <T> void swap(int a, int b, T[] arr) {
        T temp = arr[a];
        arr[a] = arr[b];
        arr[b] = temp;
    }
}
